@@ -13,10 +13,10 @@ extern void spar_constructor();
 /*--------------------------------------------------------------------- */
 
 int xc_print_sym(Clientdata *areastruct, char *sym, char*name,
-    int x, int y, int x_dis, int y_dis, int rot, float scale)
+    int x, int y, int x_dis, int y_dis, int rot)
 {
-    asg_make_instance(areastruct, sym, x, y, x_dis, y_dis, scale);
-    asg_make_label(areastruct, NORMAL, x * 15 , y * 15, name);
+    asg_make_instance(areastruct, sym, x, y, x_dis, y_dis);
+    asg_make_label(areastruct, NORMAL, x, y, name);
 
     return(1);
 }
@@ -28,7 +28,7 @@ int xc_print_sym(Clientdata *areastruct, char *sym, char*name,
 /*----------------------------------------------------------------------*/
 
 int asg_make_instance(Clientdata *clientData, char *keyword, int x,
-	int y, int x_size, int y_size , float scale1)
+	int y, int x_size, int y_size)
 {
    objectptr pobj;
    objinstptr pinst, *newinst;
@@ -57,11 +57,16 @@ int asg_make_instance(Clientdata *clientData, char *keyword, int x,
    instcopy(*newinst, pinst); 
    (*newinst)->scale = 1.0;
 
-   if (!strcmp(keyword, "arrow")) (*newinst)->rotation += 90 ;
-   else if (!strcmp(keyword, "resistor")) (*newinst)->rotation += 270 ;
-   else if (!strcmp(keyword, "inductor")) (*newinst)->rotation += 270;
-   else if (!strcmp(keyword, "capacitor")) (*newinst)->rotation += 270;
-   else if (!strcmp(keyword, "isource")) (*newinst)->rotation += 270 ;
+   // Rotated objects should be using the rotated instances.
+   // Instances at rotation = 0 in the asg_spice.lps library should
+   // match the orientations of devices in the original SPAR
+   // definitions, or vice versa.
+
+   if (!strcmp(keyword, "arrow")) (*newinst)->rotation += 90;
+   else if (!strcmp(keyword, "RESTR")) (*newinst)->rotation += 270;
+   else if (!strcmp(keyword, "INDR")) (*newinst)->rotation += 270;
+   else if (!strcmp(keyword, "CAPC")) (*newinst)->rotation += 270;
+   else if (!strcmp(keyword, "IAMP")) (*newinst)->rotation += 270;
 
    // Eventually all drawing parameters need to be overridden.
    // Assign just position for now 
