@@ -1606,19 +1606,74 @@ void finddir(polyptr lastpoly)
    }
 }
 
+/*--------------------------------------------------------------*/
+/* Edit polygon points according to cursor position (newpos)	*/ 
+/* and polygon edit style (Manhattan, Rhomboid, etc.).	The	*/
+/* original point position is returned in origpt.		*/
+/*--------------------------------------------------------------*/
+
+void boxeditadjust(polyptr newpoly, XPoint *newpos, XPoint *origpt)
+{
+   XPoint *curpt, *fpt, *bpt;
+   int deltax, deltay;
+
+   /* find the point under consideration */
+   curpt = newpoly->points + newpoly->cycle;
+
+   if (areawin->boxedit != NORMAL) {
+
+      /* find points to either side of the edit point */
+
+      fpt = bpt = NULL;
+      deltax = newpos->x - curpt->x;
+      deltay = newpos->y - curpt->y; 
+
+      if (curpt > newpoly->points) bpt = curpt - 1;
+      else if (!(newpoly->style & UNCLOSED))
+	 bpt = newpoly->points + newpoly->number - 1;
+      if (curpt < newpoly->points + newpoly->number - 1) fpt = curpt + 1;
+      else if (!(newpoly->style & UNCLOSED)) fpt = newpoly->points;
+
+      /* enforce constraints */
+
+      if (bpt != NULL) {
+	 *origpt = *bpt;
+	 if (savedir & LASTX) bpt->x += deltax;
+	 if (savedir & LASTY) bpt->y += deltay;
+
+	 if ((eventmode == EPATH_MODE) && (savedir & (LASTX | LASTY)))
+	    if (newpoly->cycle == 1)
+	       updatepath(TOPATH(EDITPART), origpt, bpt);
+      }
+      if (fpt != NULL) {
+	 *origpt = *fpt;
+	 if (savedir & NEXTX) fpt->x += deltax;
+	 if (savedir & NEXTY) fpt->y += deltay;
+
+	 if ((eventmode == EPATH_MODE) && (savedir & (NEXTX | NEXTY)))
+	    if (newpoly->cycle == newpoly->number - 2)
+	       updatepath(TOPATH(EDITPART), origpt, fpt);
+      }
+   }
+
+   /* update position of the point under consideration */
+
+   *origpt = *curpt;
+   *curpt = *newpos;
+
+   /* In case a constrained (parameterized) point has been changed, re-substitute */
+   exprsub(eventmode == EPATH_MODE ? TOGENERIC(EDITPART) : (genericptr)newpoly);
+}
+
 /*--------------------------------------------------*/
 /* Track movement of poly segments during edit mode */
 /*--------------------------------------------------*/
 
 void trackpoly(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
-   XPoint newpos, origpt, *curpt;
+   XPoint newpos, origpt;
    polyptr      newpoly;
-
-   if (eventmode == EPATH_MODE) 
-      newpoly = (polyptr)getsubpart(TOPATH(EDITPART));
-   else
-      newpoly = TOPOLY(EDITPART);
+   genericptr	*ggen;
 
    newpos = UGetCursorPos();
    u2u_snap(&newpos);
@@ -1632,55 +1687,34 @@ void trackpoly(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
    if (areawin->save.x == newpos.x && areawin->save.y == newpos.y) return;
 
+   if (eventmode == EPATH_MODE) 
+      newpoly = (polyptr)getsubpart(TOPATH(EDITPART));
+   else
+      newpoly = TOPOLY(EDITPART);
+
+   UDrawPolygon(newpoly);
+   boxeditadjust(newpoly, &newpos, &origpt);
    UDrawPolygon(newpoly);
 
-   /* find the point under consideration */
-   curpt = newpoly->points + newpoly->cycle;
+   if (areawin->multiedit) {
+      polyptr tpoly;
+      XPoint torig, dpos, *curpt, *tpt;
 
-   /* find points to either side of the edit point */
-
-   if (areawin->boxedit != NORMAL) {
-      XPoint *fpt = NULL, *bpt = NULL;
-      int deltax = newpos.x - curpt->x;
-      int deltay = newpos.y - curpt->y; 
-
-      if (curpt > newpoly->points) bpt = curpt - 1;
-      else if (!(newpoly->style & UNCLOSED))
-	 bpt = newpoly->points + newpoly->number - 1;
-      if (curpt < newpoly->points + newpoly->number - 1) fpt = curpt + 1;
-      else if (!(newpoly->style & UNCLOSED)) fpt = newpoly->points;
-
-      /* enforce constraints */
-
-      if (bpt != NULL) {
-	 origpt = *bpt;
-	 if (savedir & LASTX) bpt->x += deltax;
-	 if (savedir & LASTY) bpt->y += deltay;
-
-	 if ((eventmode == EPATH_MODE) && (savedir & (LASTX | LASTY)))
-	    if (newpoly->cycle == 1)
-	       updatepath(TOPATH(EDITPART), &origpt, bpt);
-      }
-      if (fpt != NULL) {
-	 origpt = *fpt;
-	 if (savedir & NEXTX) fpt->x += deltax;
-	 if (savedir & NEXTY) fpt->y += deltay;
-
-	 if ((eventmode == EPATH_MODE) && (savedir & (NEXTX | NEXTY)))
-	    if (newpoly->cycle == newpoly->number - 2)
-	       updatepath(TOPATH(EDITPART), &origpt, fpt);
+      curpt = newpoly->points + newpoly->cycle;
+      for (ggen = topobject->plist; ggen < topobject->plist + topobject->parts;
+		ggen++) {
+	 if (ELEMENTTYPE(*ggen) != POLYGON) continue;
+	 tpoly = TOPOLY(ggen);
+	 if (tpoly->cycle < 0) continue;
+	 if (tpoly == newpoly) continue;
+         tpt = tpoly->points + tpoly->cycle;
+	 dpos.x = tpt->x + (curpt->x - origpt.x);
+	 dpos.y = tpt->y + (curpt->y - origpt.y);
+         UDrawPolygon(tpoly);
+         boxeditadjust(tpoly, &dpos, &torig);
+         UDrawPolygon(tpoly);
       }
    }
-
-   /* update position of the point under consideration */
-
-   origpt = *curpt;
-   *curpt = newpos;
-
-   /* In case a constrained (parameterized) point has been changed, re-substitute */
-   exprsub(eventmode == EPATH_MODE ? TOGENERIC(EDITPART) : (genericptr)newpoly);
-
-   UDrawPolygon(newpoly);
    printpos(newpos.x, newpos.y);
    areawin->save.x = newpos.x;
    areawin->save.y = newpos.y;
