@@ -2298,10 +2298,17 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
    int		defaultcolor = passcolor;
    int		curcolor = passcolor;
    int		thispart;
+   short	savesel;
    XPoint 	bboxin[2], bboxout[2];
    u_char	xm, ym;
    objectptr	theobject = theinstance->thisobject;
 
+   /* Save the number of selections and set it to zero while we do the	*/
+   /* object drawing.							*/
+
+   savesel = areawin->selects;
+   areawin->selects = 0;
+
    /* All parts are given in the coordinate system of the object, unless */
    /* this is the top-level object, in which they will be interpreted as */
    /* relative to the screen.						 */
@@ -2410,6 +2417,9 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 	XTopSetForeground(passcolor);
      }
    }
+
+   /* restor the selection list (if any) */
+   areawin->selects = savesel;
    UPopCTM();
    if (stack) pop_stack(stack);
 }
@@ -2423,7 +2433,7 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 /*----------------------------------------------------------------------*/
 
 void UDoLatex(objinstptr theinstance, short level, FILE *f,
-	float scale, int tx, int ty, Boolean *checkonly)
+	float scale, float scale2, int tx, int ty, Boolean *checkonly)
 {
    XPoint	lpos, xlpos;
    XfPoint	xfpos;
@@ -2448,7 +2458,7 @@ void UDoLatex(objinstptr theinstance, short level, FILE *f,
 
       switch(ELEMENTTYPE(*areagen)) {
          case(OBJINST):
-            UDoLatex(TOOBJINST(areagen), level + 1, f, scale, tx, ty, checkonly);
+            UDoLatex(TOOBJINST(areagen), level + 1, f, scale, scale2, tx, ty, checkonly);
 	    break;
    
 	 case(LABEL): 
@@ -2474,11 +2484,19 @@ void UDoLatex(objinstptr theinstance, short level, FILE *f,
 		       xfpos.y -= 1.0;
 		       xfpos.x += 0.056;
 		       xfpos.y += 0.056;
+		       xfpos.x /= scale2;
+		       xfpos.y /= scale2;
 		       ltext = textprint(thislabel->string, theinstance);
 		       tbjust = thislabel->justify & (NOTBOTTOM | TOP);
 		       lrjust = thislabel->justify & (NOTLEFT | RIGHT);
-		       fprintf(f, "\\putbox{%3.2fin}{%3.2fin}{",
-				xfpos.x, xfpos.y);
+
+		       /* The 1.2 factor accounts for the difference between	 */
+		       /* Xcircuit's label scale of "1" and LaTeX's "normalsize" */
+
+		       fprintf(f, "   \\putbox{%3.2fin}{%3.2fin}{%3.2f}{",
+				xfpos.x, xfpos.y, 1.2 * thislabel->scale);
+		       if (thislabel->rotation != 0)
+			  fprintf(f, "\\rotatebox{-%d}{", thislabel->rotation);
 		       if (lrjust == (NOTLEFT | RIGHT)) fprintf(f, "\\rightbox{");
 		       else if (lrjust == NOTLEFT) fprintf(f, "\\centbox{");
 		       if (tbjust == (NOTBOTTOM | TOP)) fprintf(f, "\\topbox{");
@@ -2486,6 +2504,7 @@ void UDoLatex(objinstptr theinstance, short level, FILE *f,
 		       fprintf(f, "%s", ltext);
 		       if (lrjust != NORMAL) fprintf(f, "}");
 		       if (tbjust != NORMAL) fprintf(f, "}");
+		       if (thislabel->rotation != 0) fprintf(f, "}");
 		       fprintf(f, "}%%\n");
 		       free(ltext);
 		    }
@@ -2503,13 +2522,13 @@ void UDoLatex(objinstptr theinstance, short level, FILE *f,
 void TopDoLatex()
 {
    FILE *f;
-   float psscale;
+   float psscale, outscale;
    int tx, ty, width, height;
    XPoint origin;
    Boolean checklatex = FALSE;
    char filename[100], extend[10], *dotptr;
 
-   UDoLatex(areawin->topinstance, 0, NULL, 1.0, 0, 0, &checklatex);
+   UDoLatex(areawin->topinstance, 0, NULL, 1.0, 1.0, 0, 0, &checklatex);
 
    if (checklatex == FALSE) return;	/* No LaTeX labels to write */
 
@@ -2535,19 +2554,20 @@ void TopDoLatex()
 
    fprintf(f, "%% XCircuit output \"%s.tex\" for LaTeX input from %s%s\n",
 		filename, filename, extend);
-   fprintf(f, "\\def\\putbox#1#2#3{\\makebox[0in][l]{\\makebox[#1][l]{}"
+   fprintf(f, "\\def\\putbox#1#2#3#4{\\makebox[0in][l]{\\makebox[#1][l]{}"
 		"\\raisebox{\\baselineskip}[0in][0in]"
-		"{\\raisebox{#2}[0in][0in]{#3}}}}\n");
+		"{\\raisebox{#2}[0in][0in]{\\scalebox{#3}{#4}}}}}\n");
    fprintf(f, "\\def\\rightbox#1{\\makebox[0in][r]{#1}}\n");
    fprintf(f, "\\def\\centbox#1{\\makebox[0in]{#1}}\n");
-   fprintf(f, "\\def\\topbox#1{\\raisebox{-\\baselineskip}[0in][0in]{#1}}\n");
-   fprintf(f, "\\def\\midbox#1{\\raisebox{-0.5\\baselineskip}[0in][0in]{#1}}\n");
-   fprintf(f, "\\begin{flushleft}\n");
-   
-   fprintf(f, "\\epsfig{file=%s%s}\\\\\n", filename, extend);
+   fprintf(f, "\\def\\topbox#1{\\raisebox{-0.60\\baselineskip}[0in][0in]{#1}}\n");
+   fprintf(f, "\\def\\midbox#1{\\raisebox{-0.20\\baselineskip}[0in][0in]{#1}}\n");
 
-   psscale = getpsscale(xobjs.pagelist[areawin->page]->outscale,
-		areawin->page);
+   /* Modified to use \scalebox and \parbox by Alex Tercete, June 2008 */
+
+   fprintf(f, "\\begin{center}\n");
+
+   outscale = xobjs.pagelist[areawin->page]->outscale;
+   psscale = getpsscale(outscale, areawin->page);
 
    width = toplevelwidth(areawin->topinstance, &origin.x);
    height = toplevelheight(areawin->topinstance, &origin.y);
@@ -2555,14 +2575,23 @@ void TopDoLatex()
    tx = (int)(72 / psscale) - origin.x,
    ty = (int)(72 / psscale) - origin.y;
 
-   fprintf(f, "%% translate x=%d y=%d scale %3.2f\n", tx, ty, psscale);
+   fprintf(f, "   \\scalebox{%g}{%\n", outscale);
+   fprintf(f, "   \\normalsize\n");
+   fprintf(f, "   \\parbox{%gin}{%\n", (((float)width * psscale) / 72.0) / outscale);
+   fprintf(f, "   \\includegraphics[scale=%g]{%s%s}\\\\\n", 1.0 / outscale,
+			filename, extend);
+   fprintf(f, "   %% translate x=%d y=%d scale %3.2f\n", tx, ty, psscale);
 
    UPushCTM();		/* Save current state */
    UResetCTM(DCTM);	/* Set to identity matrix */
-   UDoLatex(areawin->topinstance, 0, f, psscale, tx, ty, NULL);
+   UDoLatex(areawin->topinstance, 0, f, psscale, outscale, tx, ty, NULL);
    UPopCTM();		/* Restore state */
 
-   fprintf(f, "\\end{flushleft}\n");
+   fprintf(f, "   } %% close \'parbox\'\n");
+   fprintf(f, "   } %% close \'scalebox\'\n");
+   fprintf(f, "   \\vspace{-\\baselineskip} %% this is not"
+		" necessary, but looks better\n");
+   fprintf(f, "\\end{center}\n");
    fclose(f);
 
    Wprintf("Wrote auxiliary file %s.tex", filename);
