@@ -1741,8 +1741,10 @@ void startcatalog(xcWidget w, pointertype libmod, caddr_t nulldata)
       XDefineCursor (dpy, areawin->window, DEFAULTCURSOR);
       eventmode = CATALOG_MODE;
    }
-   else if (eventmode != CATMOVE_MODE)
+   else if (eventmode != CATMOVE_MODE) {
+      finish_op(XCF_Cancel, 0, 0);
       eventmode = CATALOG_MODE;
+   }
 
    /* Push the current page onto the push stack, unless	we're going */
    /* to a library from the library directory or vice versa, or	    */
