@@ -211,6 +211,7 @@ void changetextscale(float newscale)
 
    else if (areawin->selects > 0) {
       float oldscale;
+      Boolean waschanged = FALSE;
       for (osel = areawin->selectlist; osel < areawin->selectlist +
 	     areawin->selects; osel++) {
 	 if (SELECTTYPE(osel) == LABEL) {
@@ -222,9 +223,11 @@ void changetextscale(float newscale)
 	       redrawtext(settext);
 	       register_for_undo(XCF_Rescale, UNDO_MORE, areawin->topinstance,
 			(genericptr)settext, (double)oldscale);
+	       waschanged = TRUE;
 	    }
 	 }
       }
+      if (waschanged) undo_finish_series();
    }
 }
 
@@ -470,6 +473,7 @@ void setdscale(xcWidget w, XPoint *dataptr)
 void setosize(xcWidget w, objinstptr dataptr)
 {
    float tmpres, oldsize;
+   Boolean waschanged = FALSE;
    short *osel;
    objinstptr nsobj;
    int res = sscanf(_STR2, "%f", &tmpres);
@@ -485,12 +489,15 @@ void setosize(xcWidget w, objinstptr dataptr)
 	 oldsize = nsobj->scale;
          nsobj->scale = tmpres;
 
-         if (oldsize != tmpres)
+         if (oldsize != tmpres) {
             register_for_undo(XCF_Rescale, UNDO_MORE, areawin->topinstance,
 			SELTOGENERIC(osel), (double)oldsize);
+	    waschanged = TRUE;
+	 }
       }
    }
-   unselect_all();
+   /* unselect_all(); */
+   if (waschanged) undo_finish_series();
    pwriteback(areawin->topinstance);
    drawarea(NULL, NULL, NULL);
 }
