@@ -7,21 +7,32 @@
 /* We use OpenGL functions to read and scale images if OpenGL is enabled */
 #ifndef OPENGL
 
-#undef GS_DEBUG
-/* #define GS_DEBUG */
+/* #undef GS_DEBUG */
+#define GS_DEBUG
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#ifndef XC_WIN32
 #include <unistd.h>
+#endif
 #include <time.h>
+#ifndef XC_WIN32
 #include <sys/wait.h>   /* for waitpid() */
+#endif
 #include <sys/types.h>
 #include <signal.h>
 #include <fcntl.h>
+
+#ifndef XC_WIN32
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/Xatom.h>
+#else
+#ifdef TCL_WRAPPER
+#include <X11/Xatom.h>
+#endif
+#endif
 
 /*------------------------------------------------------------------------*/
 /* Local includes                                                         */
@@ -72,7 +83,11 @@ int gs_state;		/* Track state of the renderer */
 /*------------------------------------------------------*/
 
 Atom  gv, gvc, gvpage, gvnext, gvdone;
+#ifndef _MSC_VER
 pid_t gsproc = -1;	/* ghostscript process 			*/
+#else
+HANDLE gsproc = INVALID_HANDLE_VALUE;
+#endif
 int   fgs[2];		/* stdin pipe pair for ghostscript	*/
 Window mwin = 0;	/* "false" window hack to get gs	*/
 			/* process to capture ClientMessage	*/
@@ -265,6 +280,58 @@ void start_gs()
    /* the background file to gs for rendering.				*/
    /* Here is not the place to do it.  Set up gs to take stdin as input.*/
 
+#ifdef _MSC_VER
+   if (gsproc == INVALID_HANDLE_VALUE) {
+	 STARTUPINFO st_info;
+	 PROCESS_INFORMATION pr_info;
+	 char cmd[4096];
+
+#ifdef HAVE_PUTENV
+   	 sprintf(env_str1, "DISPLAY=%s", XDisplayString(dpy));
+	 putenv(env_str1);
+#ifdef DOUBLEBUFFER
+   	 sprintf(env_str2, "GHOSTVIEW=%ld %ld", (long)areawin->window, (long)bbuf);
+	 putenv(env_str2);
+#else
+	 sprintf(env_str2, "GHOSTVIEW=%ld", (long)areawin->window);
+	 putenv(env_str2);
+#endif
+#else
+	 setenv("DISPLAY", XDisplayString(dpy), True);
+#ifdef DOUBLEBUFFER
+	 sprintf(_STR, "%ld %ld", (long)areastruct.areawin, (long)bbuf);
+	 setenv("GHOSTVIEW", _STR, True);
+#else
+ 	 sprintf(_STR, "%ld", (long)areastruct.areawin);
+	 setenv("GHOSTVIEW", _STR, True);
+#endif
+#endif
+
+	 SetHandleInformation((HANDLE)_get_osfhandle(fgs[1]), HANDLE_FLAG_INHERIT, 0);
+	 SetHandleInformation((HANDLE)_get_osfhandle(std_out[0]), HANDLE_FLAG_INHERIT, 0);
+#ifndef GS_DEBUG
+	 SetHandleInformation((HANDLE)_get_osfhandle(std_err[0]), HANDLE_FLAG_INHERIT, 0);
+#endif
+
+	 ZeroMemory(&st_info, sizeof(STARTUPINFO));
+	 ZeroMemory(&pr_info, sizeof(PROCESS_INFORMATION));
+	 st_info.cb = sizeof(STARTUPINFO);
+	 st_info.dwFlags = STARTF_USESTDHANDLES;
+	 st_info.hStdOutput = (HANDLE)_get_osfhandle(std_out[1]);
+	 st_info.hStdInput = (HANDLE)_get_osfhandle(fgs[0]);
+#ifndef GS_DEBUG
+	 st_info.hStdError = (HANDLE)_get_osfhandle(std_err[1]);
+#endif
+
+	 snprintf(cmd, 4095, "%s -dNOPAUSE -", GS_EXEC);
+	 if (CreateProcess(NULL, cmd, NULL, NULL, TRUE, 0, NULL, NULL, &st_info, &pr_info) == 0) {
+		 Wprintf("Error: ghostscript not running");
+		 return;
+	 }
+	 CloseHandle(pr_info.hThread);
+	 gsproc = pr_info.hProcess;
+   }
+#else
    if (gsproc < 0) {  /* Ghostscript is not running yet */
       gsproc = vfork();
       if (gsproc == 0) {		/* child process (gs) */
@@ -315,6 +382,7 @@ void start_gs()
 	 return;			/* error condition */
       }
    }
+#endif
 }
 
 /*--------------------------------------------------------*/
@@ -429,7 +497,11 @@ void readbackground(FILE *fi)
    /* "@" denotes a temporary file */
    sprintf(file_in, "@%s/XXXXXX", xobjs.tempdir);
 
+#ifdef _MSC_VER
+   tfd = mktemp(file_in + 1);
+#else
    tfd = mkstemp(file_in + 1);
+#endif
    if (tfd == -1) fprintf(stderr, "Error generating temporary filename\n");
    else {
       if ((fbg = fdopen(tfd, "w")) == NULL) {
@@ -627,19 +699,31 @@ int copybackground()
 
 int exit_gs()
 {
+#ifdef _MSC_VER
+   if (gsproc == INVALID_HANDLE_VALUE) return -1;	/* gs not running */
+#else
    if (gsproc < 0) return -1;	/* gs not running */
+#endif
 
 #ifdef GS_DEBUG
    fprintf(stdout, "Waiting for gs to exit\n");
 #endif
+#ifndef _MSC_VER
    kill(gsproc, SIGKILL);
    waitpid(gsproc, NULL, 0);      
+#else
+   TerminateProcess(gsproc, -1);
+#endif
 #ifdef GS_DEBUG
    fprintf(stdout, "gs has exited\n");
 #endif
 
    mwin = 0;
+#ifdef _MSC_VER
+   gsproc = INVALID_HANDLE_VALUE;
+#else
    gsproc = -1;
+#endif
    gs_state = GS_INIT;
 
    return 0;
@@ -651,7 +735,11 @@ int exit_gs()
 
 int reset_gs()
 {
+#ifdef _MSC_VER
+   if (gsproc == INVALID_HANDLE_VALUE) return -1;
+#else
    if (gsproc < 0) return -1;
+#endif
 
    exit_gs();
    ghostinit_local();
