@@ -103,6 +103,18 @@ objectptr NameToPageObject(char *objname, objinstptr *ret_inst, int *ret_page)
    return NULL;
 }
 
+/*--------------------------------------------------------------*/
+/* Get the canonical name of an object (the part without the	*/
+/* technology prefix, if any).					*/
+/*--------------------------------------------------------------*/
+
+char *GetCanonicalName(char *fullname)
+{
+   char *canonname = strstr(fullname, "::");
+   if (canonname == NULL) return fullname;
+   return canonname + 2;
+}
+
 /*----------------------------------------------------------------------*/
 /* Find the object with the indicated name.				*/
 /*									*/
@@ -124,13 +136,8 @@ objectptr NameToObject(char *objname, objinstptr *ret_inst, Boolean dopages)
    for (i = 0; i < xobjs.numlibs; i++) {
       for (spec = xobjs.userlibs[i].instlist; spec != NULL; spec = spec->next) {
 	 techptr = spec->thisinst->thisobject->name;
-	 if (notech) {
-	    techptr = strstr(spec->thisinst->thisobject->name, "::");
-	    if (techptr == NULL)
-	       techptr = spec->thisinst->thisobject->name;
- 	    else
-	       techptr += 2;
-	 }
+	 if (notech)
+	    techptr = GetCanonicalName(spec->thisinst->thisobject->name);
          if (!strcmp(objname, techptr)) {
 	    if (ret_inst) *ret_inst = spec->thisinst;
 	    return spec->thisinst->thisobject;
@@ -183,7 +190,7 @@ int checkpagename(objectptr thispageobj)
       for (p = 0; p < xobjs.pages; p++) {
 	 if (p == thispage) continue;
          if (xobjs.pagelist[p]->pageinst != NULL) {
-	    if (!strcmp(xobjs.pagelist[p]->pageinst->thisobject->name,
+	    if (!filecmp(xobjs.pagelist[p]->pageinst->thisobject->name,
 			thispageobj->name)) {
 	       /* append ":2" to name or update suffix to ensure uniqueness */
 	       if (clnptr == NULL)
@@ -470,14 +477,7 @@ void dopintype(xcWidget w, pointertype mode, caddr_t calldata)
 
 void setsymschem()
 {
-   /* This is temporary (hopefully). . . Call the Tcl function	*/
-   /* "setsymschem" which had better be defined by the wrapper.	*/
-
-   Tcl_SavedResult state;
-
-   Tcl_SaveResult(xcinterp, &state);
-   Tcl_Eval(xcinterp, "xcircuit::setsymschem");
-   Tcl_RestoreResult(xcinterp, &state);
+   XcInternalTagCall(xcinterp, 1, "schematic");
 }
 
 #else
@@ -582,9 +582,19 @@ int findpageobj(objectptr pobj)
 /* number of recursion levels.  Presumably no circuit	*/
 /* would have more than several hundred hierarchical 	*/
 /* levels.						*/
+/*							*/
+/* If "dolinks" is TRUE, ennumerate and follow pages	*/
+/* that are descendents of the top level when the	*/
+/* calling symbol instance has a "link" parameter and	*/
+/* that parameter points to the same filename as the	*/
+/* page.  This indicates a multiple-file project; the	*/
+/* pages keep separate records of changes, and are	*/
+/* saved independently, and loaded through the link	*/
+/* dependency method.					*/
 /*------------------------------------------------------*/
 
-int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
+int findsubschems(int toppage, objectptr cschem, int level, short *pagelist,
+	Boolean dolinks)
 {
    genericptr *cgen;
 
@@ -592,12 +602,33 @@ int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
 
    for (cgen = cschem->plist; cgen < cschem->plist + cschem->parts; cgen++) {
       if (IS_OBJINST(*cgen)) {
-	 objectptr cobj = TOOBJINST(cgen)->thisobject;
+	 objinstptr cinst = TOOBJINST(cgen);
+	 objectptr cobj = cinst->thisobject;
 
 	 if (cobj->symschem != NULL) {
 	    int pageno = findpageobj(cobj->symschem);
-	    if ((pageno >= 0) && (pageno < xobjs.pages))
-		pagelist[pageno]++;
+
+	    if ((pageno >= 0) && (pageno < xobjs.pages)) {
+
+	       /* Look for a "link" parameter */
+	       if (dolinks == FALSE) {
+		  oparamptr ops;
+	          ops = find_param(cinst, "link");
+	          if ((ops != NULL) && (ops->type == XC_STRING)) {
+	             char *filename = textprint(ops->parameter.string, cinst);
+		     if (!strcmp(filename, "%n") || !strcmp(filename, "%N") ||
+			  ((xobjs.pagelist[pageno]->filename != NULL)  &&
+			  (!strcmp(filename, xobjs.pagelist[pageno]->filename)))) {
+		        free(filename);
+		        continue;
+		     }
+	             free(filename);
+		  }
+	       }
+
+	       /* Add this page to the list */
+	       pagelist[pageno]++;
+	    }
 
 	    /* A symbol on its own schematic page is allowed for clarity */
 	    /* of the schematic, but this cannot be a functional part of */
@@ -605,13 +636,13 @@ int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
 	    
 	    if (cobj->symschem != cschem) {
 	       if (findsubschems(toppage, cobj->symschem,
-				level + 1, pagelist) == -1)
+				level + 1, pagelist, dolinks) == -1)
 		  return -1;
 	    }
 	 }
 	 else if (cobj->schemtype != FUNDAMENTAL && cobj->schemtype != TRIVIAL) {
 	    /* Check symbols acting as their own schematics */
-	    if (findsubschems(toppage, cobj, level + 1, pagelist) == -1)
+	    if (findsubschems(toppage, cobj, level + 1, pagelist, dolinks) == -1)
 	       return -1;
 	 }
       }
@@ -619,15 +650,16 @@ int findsubschems(int toppage, objectptr cschem, int level, short *pagelist)
    return 0;
 }
 
-/*-------------------------------------------------------*/
-/* Recursively find all sub-circuits associated with the */
-/* top-level circuit and set their filenames to be the	 */
-/* same.						 */
-/*-------------------------------------------------------*/
+/*------------------------------------------------------*/
+/* Recursively find all sub-circuits associated with	*/
+/* the top-level circuit.  For each schematic that does	*/
+/* not have a valid filename, set the filename equal to	*/
+/* the filename of the "toppage" schematic.		*/
+/*------------------------------------------------------*/
 
 void collectsubschems(int toppage)
 {
-  int loctop; /* , level = 0; (jdk) */
+   int loctop;
    objectptr cschem;
    short *pagelist, pageno;
    Pagedata *curpage;
@@ -648,15 +680,15 @@ void collectsubschems(int toppage)
    for (pageno = 0; pageno < xobjs.pages; pageno++)
       pagelist[pageno] = 0;
 
-   findsubschems(loctop, cschem, 0, pagelist);
+   findsubschems(loctop, cschem, 0, pagelist, FALSE);
 
    for (pageno = 0; pageno < xobjs.pages; pageno++) {
       if (pageno == loctop) continue;
       if (pagelist[pageno] > 0) {
-	  if (xobjs.pagelist[pageno]->filename != NULL)
-	     free(xobjs.pagelist[pageno]->filename);
-	  xobjs.pagelist[pageno]->filename =
-	     strdup(xobjs.pagelist[loctop]->filename);
+	 if (xobjs.pagelist[pageno]->filename != NULL)
+	    free(xobjs.pagelist[pageno]->filename);
+	 xobjs.pagelist[pageno]->filename =
+	    strdup(xobjs.pagelist[loctop]->filename);
       }
    } 
    free((char *)pagelist);
@@ -757,15 +789,19 @@ int checksym(objectptr symobj, char *cname)
 /*----------------------------------------------------------------------*/
 /* Find location of corresponding pin in symbol/schematic and change it */
 /* to the text of the indicated label.					*/
+/*									*/
+/* Return the number of other pins changed.  zero indicates that no	*/
+/* corresponding pins were found, and therefore nothing was changed.	*/
 /*----------------------------------------------------------------------*/
 
-void changeotherpins(labelptr newlabel, stringpart *oldstring)
+int changeotherpins(labelptr newlabel, stringpart *oldstring)
 {
    objectptr other = topobject->symschem;
    genericptr *tgen;
    labelptr tlab;
+   int rval = 0;
 
-   if (other == NULL) return;
+   if (other == NULL) return rval;
 
    for (tgen = other->plist; tgen < other->plist + other->parts; tgen++) {
       if (IS_LABEL(*tgen)) {
@@ -775,10 +811,12 @@ void changeotherpins(labelptr newlabel, stringpart *oldstring)
 	    if (newlabel != NULL) {
 	       free(tlab->string);
 	       tlab->string = stringcopy(newlabel->string);
+	       rval++;
 	    }
 	 }
       }
    }
+   return rval;
 }
 
 /*----------------------------------------------------------------------*/
@@ -787,7 +825,7 @@ void changeotherpins(labelptr newlabel, stringpart *oldstring)
 
 void xlib_swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 {
-   swapschem((int)mode, -1);
+   swapschem((int)mode, -1, NULL);
 }
 
 /*----------------------------------------------------------------------*/
@@ -797,7 +835,7 @@ void xlib_swapschem(xcWidget w, pointertype mode, caddr_t calldata)
 /*  happens.								*/
 /*----------------------------------------------------------------------*/
 
-void swapschem(int allow_create, int libnum)
+void swapschem(int allow_create, int libnum, char *fullname)
 {
    objectptr savepage = topobject;
    labelptr  *pinlab;
@@ -805,16 +843,15 @@ void swapschem(int allow_create, int libnum)
    Boolean lflag;
    pushlistptr stacktop;
    short loclibnum = libnum;
+   char *canonname;
 
    if (libnum == -1) loclibnum = USERLIB - LIBRARY;
 
-   if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
-      standard_element_delete(NORMAL);
-
    /* Create symbol or schematic, if allowed by allow_create */
 
    if ((topobject->symschem == NULL) && (allow_create != 0)
 		&& (topobject->schemtype != SECONDARY)) {
+
       if (topobject->schemtype != PRIMARY) {
 	 int tpage;
 
@@ -849,6 +886,9 @@ void swapschem(int allow_create, int libnum)
 
 	 incr_changes(*newobject);
 
+	 if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
+	    standard_element_delete(ERASE);
+
 	 /* Generate a library instance for this object and set the */
 	 /* top instance to point to it.			    */
 
@@ -866,9 +906,21 @@ void swapschem(int allow_create, int libnum)
       savepage->symschem = topobject;
       topobject->symschem = savepage;
 
-      /* make the name of the new object equal to that of the old */
+      /* Make the name of the new object equal to that of the old, */
+      /* except that symbols get the full name while schematics	   */
+      /* get the canonical name (without the technology prefix)	   */
 
-      strcpy(topobject->name, savepage->name);
+      if (fullname == NULL)
+	 canonname = GetCanonicalName(savepage->name);
+      else {
+         canonname = strstr(fullname, "::");
+	 if ((canonname == NULL) || (topobject->schemtype != PRIMARY))
+	    canonname = fullname;
+	 else
+	    canonname += 2;
+      }
+      strcpy(topobject->name, canonname);
+      checkname(topobject);
 
       /* copy all pin labels into the new object */
 
@@ -959,6 +1011,9 @@ void swapschem(int allow_create, int libnum)
 	    }
          }
 	       
+	 if (eventmode == MOVE_MODE || eventmode == COPY_MODE)
+	    delete_for_xfer(NORMAL, areawin->selectlist, areawin->selects);
+
 	 topobject->viewscale = areawin->vscale;
 	 topobject->pcorner = areawin->pcorner;
 	 push_stack(&areawin->stack, areawin->topinstance);
@@ -986,9 +1041,7 @@ void makesymbol(xcWidget w, caddr_t calldata)
 {
    /* copy name from popup prompt buffer and check */
 
-   strcpy(topobject->name, _STR2);
-   checkname(topobject);
-   swapschem(1, -1);
+   swapschem(1, -1, _STR2);
 }
 
 /*----------------------------------------------------------------------*/
@@ -1013,7 +1066,7 @@ void dobeforeswap(xcWidget w, caddr_t clientdata, caddr_t calldata)
       popupprompt(w, "Enter name for new object:", "\0", makesymbol, popdata, NULL);
    }
    else
-      swapschem(1, -1);
+      swapschem(1, -1, NULL);
 }
 
 #endif /* !TCL_WRAPPER */
@@ -1088,7 +1141,8 @@ Boolean schemassoc(objectptr schemobj, objectptr symbolobj)
 	 symbolobj->schemtype = SYMBOL;
 
       /* Schematic takes the name of its associated symbol, by default */
-      strcpy(schemobj->name, symbolobj->name);
+      /* Don't copy any technology prefix. */
+      strcpy(schemobj->name, GetCanonicalName(symbolobj->name));
 
       /* Ensure that schematic (page) name is unique */
       while (checkpagename(schemobj) < 0);
