@@ -858,7 +858,7 @@ void freeselects()
 void clearselects_noundo()
 {
    if (areawin->selects > 0) {
-      reset_connected_pins();
+      if (areawin->multiedit != True) reset_connected_pins();
       freeselects();
       if (xobjs.suspend < 0) {
          setallstylemarks(areawin->style);
