@@ -578,6 +578,9 @@ void printeventmode() {
       case EPATH_MODE:
 	 Fprintf(stderr, "EPATH");
 	 break;
+      case EINST_MODE:
+	 Fprintf(stderr, "EINST");
+	 break;
       case ASSOC_MODE:
 	 Fprintf(stderr, "ASSOC");
 	 break;
@@ -1066,6 +1069,7 @@ XCWindowData *create_new_window()
    newwindow->defaultcursor = &CROSS;
    newwindow->event_mode = NORMAL_MODE;
    newwindow->attachto = -1;
+   newwindow->multiedit = False;
    newwindow->color = DEFAULTCOLOR;
    newwindow->gccolor = 0;
    newwindow->gctype = 0;
@@ -1130,6 +1134,8 @@ void pre_initialize()
    xobjs.pagelist[0]->coordstyle = INTERNAL;
    xobjs.pagelist[0]->pagesize.x = 612;
    xobjs.pagelist[0]->pagesize.y = 792;
+   xobjs.pagelist[0]->margins.x = 72;
+   xobjs.pagelist[0]->margins.y = 72;
 
    xobjs.hold = TRUE;
    xobjs.showtech = FALSE;
